package com.example.springdemo.security;

import org.jetbrains.annotations.NotNull;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.Customizer;
import org.springframework.security.config.annotation.authentication.configuration.EnableGlobalAuthentication;
import org.springframework.security.config.annotation.method.configuration.EnableMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.config.annotation.web.configurers.LogoutConfigurer;
import org.springframework.security.web.SecurityFilterChain;

@Configuration
@EnableWebSecurity // Enable Spring Security
@EnableGlobalAuthentication // Enable Spring Security's global authentication configuration
@EnableMethodSecurity(prePostEnabled = true) // Enable Spring Security's method security
public class SecurityConfig {
    @Bean
    public SecurityFilterChain SecurityFilterChain(@NotNull HttpSecurity http) throws Exception {
        var ignoreUrls = new String[]{"/login", "/logout", "/error"};
        var authedUrls = new String[]{"/users/*/**"};
        http
                .authorizeHttpRequests(
                        (request) -> request
                                .requestMatchers(authedUrls).authenticated() // authenticate all requests to authedUrls
                                .requestMatchers(ignoreUrls).permitAll() // permit all requests to ignoreUrls
                )
                .formLogin(Customizer.withDefaults())
                .httpBasic(Customizer.withDefaults())
                .csrf(AbstractHttpConfigurer::disable)
                .logout(LogoutConfigurer::permitAll);
        return http.build();
    }
}
